package Language::INTERCAL::ByteCode;

# Definitions of bytecode symbols etc

# This file is part of CLC-INTERCAL

# Copyright (c) 2006 Claudio Calvelli, all rights reserved.

# CLC-INTERCAL is copyrighted software. However, permission to use, modify,
# and distribute it is granted provided that the conditions set out in the
# licence agreement are met. See files README and COPYING in the distribution.


use strict;
use vars qw($PERVERSION);
$PERVERSION = "CLC-INTERCAL INTERCAL/ByteCode.pm 1.-94.-4";

use Carp;

use Language::INTERCAL::Exporter '1.-94.-4';
use Language::INTERCAL::Splats '1.-94.-4', qw(:SP);
use Language::INTERCAL::Numbers '1.-94.-4';
use Language::INTERCAL::DoubleOhSeven '1.-94.-4';
use Language::INTERCAL::SharkFin '1.-94.-4';
use Language::INTERCAL::Arrays '1.-94.-4';
use Language::INTERCAL::Whirlpool '1.-94.-4';
use Language::INTERCAL::GenericIO '1.-94.-4',
	qw($stdwrite $stdread $stdsplat $devnull);

use vars qw(@EXPORT @EXPORT_OK %EXPORT_TAGS);

use constant BYTE_SIZE     => 8;      # number of bits per byte (must be == 8)
use constant NUM_OPCODES   => 0x80;   # number of virtual opcodes
use constant OPCODE_RANGE  => 1 << BYTE_SIZE;
use constant BC_MASK       => OPCODE_RANGE - 1;
use constant BIGNUM_SHIFT  => BYTE_SIZE - 1;
use constant BIGNUM_RANGE  => 1 << BIGNUM_SHIFT;
use constant BIGNUM_MASK   => (BIGNUM_RANGE - 1) << 1;
use constant BYTE_SHIFT    => OPCODE_RANGE - NUM_OPCODES;

@EXPORT_OK = qw(
    bytecode bytedecode bc_list BC BCget bc_bytype bc_match BC_MASK
    BC_constants is_constant is_multibyte bc_xtype bc_skip bc_forall
    BC_ABG BC_ABL BC_AWC BC_BAW BC_BBT BC_BSW BC_BUG BC_BUT BC_BWC BC_CFG
    BC_CFL BC_CHO BC_CON BC_CRE BC_CWB BC_DES BC_DOS BC_DSX BC_EBC BC_ECB
    BC_ENR BC_ENS BC_FIN BC_FOR BC_FRE BC_FRZ BC_GRA BC_GUP BC_HSN BC_HYB
    BC_IGN BC_INT BC_LAB BC_LEA BC_MSP BC_MUL BC_NOT BC_NUM BC_NXG BC_NXL
    BC_NXT BC_OPT BC_OSN BC_OVM BC_OVR BC_OWN BC_QUA BC_REG BC_REL BC_REM
    BC_RES BC_RET BC_ROM BC_ROR BC_ROU BC_SEL BC_SHF BC_SPL BC_SPO BC_STA
    BC_STO BC_STR BC_STS BC_STU BC_SUB BC_SWA BC_SWB BC_SYS BC_TAI BC_TSP
    BC_TYP BC_UDV BC_UNE BC_UNS BC_WHP BC_WIN
    reg_list reg_name reg_create reg_codetype reg_decode
    reg_code
);

%EXPORT_TAGS = (
    BC => [qw(
	BC BCget BC_MASK bytecode bytedecode
	BC_ABG BC_ABL BC_AWC BC_BAW BC_BBT BC_BSW BC_BUG BC_BUT BC_BWC
	BC_CFG BC_CFL BC_CHO BC_CON BC_CRE BC_CWB BC_DES BC_DOS BC_DSX
	BC_EBC BC_ECB BC_ENR BC_ENS BC_FIN BC_FOR BC_FRE BC_FRZ BC_GRA
	BC_GUP BC_HSN BC_HYB BC_IGN BC_INT BC_LAB BC_LEA BC_MSP BC_MUL
	BC_NOT BC_NUM BC_NXG BC_NXL BC_NXT BC_OPT BC_OSN BC_OVM BC_OVR
	BC_OWN BC_QUA BC_REG BC_REL BC_REM BC_RES BC_RET BC_ROM BC_ROR
	BC_ROU BC_SEL BC_SHF BC_SPL BC_SPO BC_STA BC_STO BC_STR BC_STS
	BC_STU BC_SUB BC_SWA BC_SWB BC_SYS BC_TAI BC_TSP BC_TYP BC_UDV
	BC_UNE BC_UNS BC_WHP BC_WIN
    )],
);

my %bytecodes = (
    ABG => ['ABstain from Gerund', 'S', '15', 'C(O)', 0, 0],
    ABL => ['ABstain from Label', 'S', '14', 'E', 0, 0],
    AWC => ['unary Add Without Carry', 'E', '102', 'E', 0, 1],
    BAW => ['binary Add Without Carry', 'E', '103', 'EE', 0, 1],
    BBT => ['binary BUT', 'E', '99', '#EE', 0, 1],
    BSW => ['binary Subtract Without Borrow', 'E', '101', 'EE', 0, 1],
    BUG => ['compiler BUG', 'S', '38', '#', 0, 0],
    BUT => ['unary BUT', 'E', '98', '#E', 0, 1],
    BWC => ['loop: Body While Condition', 'S', '25', 'SS', 0, 0],
    CFG => ['Come From Gerund', 'S', '23', 'C(O)', 0, 0],
    CFL => ['Come From Label', 'S', '22', 'E', 0, 0],
    CHO => ['Crawling HOrror', 'R', '71', 'E', 0, 1],
    CON => ['CONvert', 'S', '36', 'OO', 0, 0],
    CRE => ['CREate', 'S', '2', 'EVC(<)C(>)', 0, 0],
    CWB => ['loop: Condition While Body', 'S', '26', 'SS', 0, 0],
    DES => ['DEStroy', 'S', '3', 'EVC(<)', 0, 0],
    DOS => ['Double-Oh-Seven', 'R', '69', 'E', 0, 1],
    DSX => ['Double-oh-Seven eXecution', 'S', '5', 'ES', 0, 0],
    EBC => ['Event: Body while Condition', 'S', '40', 'ES', 0, 0],
    ECB => ['Event: Condition while Body', 'S', '41', 'ES', 0, 0],
    ENR => ['ENRol', 'S', '30', 'C(E)R', 0, 0],
    ENS => ['ENSlave', 'S', '27', 'RR', 0, 0],
    FIN => ['FINish lecture', 'S', '32', '', 0, 0],
    FOR => ['FORget', 'S', '9', 'E', 0, 0],
    FRE => ['FREe', 'S', '28', 'RR', 0, 0],
    FRZ => ['FReeZe', 'S', '42', '', 0, 0],
    GRA => ['GRAduate', 'S', '33', 'R', 0, 0],
    GUP => ['Give UP', 'S', '18', '', 0, 0],
    HSN => ['Half Spot Number', '#', '126', 'N', 1, 1],
    HYB => ['HYBrid', 'R', '67', 'E', 0, 1],
    IGN => ['IGNore', 'S', '12', 'C(R)', 0, 0],
    INT => ['INTerleave', 'E', '105', 'EE', 0, 1],
    LAB => ['LABel', 'S', '21', 'ES', 0, 0],
    LEA => ['LEArns', 'S', '31', 'ER', 0, 0],
    MSP => ['Make SPlat', 'S', '4', 'EC(V)', 0, 0],
    MUL => ['MULtiple number', 'E', '96', 'C(E)', 0, 0],
    NOT => ['NOT', 'S', '6', 'S', 0, 0],
    NUM => ['NUMber', 'E', '106', 'R', 0, 1],
    NXG => ['Next From Gerund', 'S', '35', 'C(O)', 0, 0],
    NXL => ['Next From Label', 'S', '34', 'E', 0, 0],
    NXT => ['NeXT', 'S', '7', 'E', 0, 0],
    OPT => ['OPTimise', 'S', '39', 'C([)C(])', 0, 0],
    OSN => ['One Spot Number', '#', '127', 'NN', 1, 1],
    OVM => ['OVerload Many', 'E', '107', 'EE', 0, 1],
    OVR => ['OVerload Register', 'R', '80', 'ER', 0, 1],
    OWN => ['OWNer', 'R', '82', 'ER', 0, 1],
    QUA => ['QUAntum statement', 'S', '24', 'S', 0, 0],
    REG => ['REinstate from Gerund', 'S', '17', 'C(O)', 0, 0],
    REL => ['REinstate from Label', 'S', '16', 'E', 0, 0],
    REM => ['REMember', 'S', '13', 'C(R)', 0, 0],
    RES => ['RESume', 'S', '8', 'E', 0, 0],
    RET => ['RETrieve', 'S', '11', 'C(R)', 0, 0],
    ROM => ['Remove Overload Many', 'E', '108', 'E', 0, 1],
    ROR => ['Remove Overload Register', 'R', '81', 'R', 0, 1],
    ROU => ['Read OUt', 'S', '20', 'C(E)', 0, 0],
    SEL => ['SELect', 'E', '104', 'EE', 0, 1],
    SHF => ['SHark Fin', 'R', '70', 'E', 0, 1],
    SPL => ['SPLat', 'E', '109', '', 0, 1],
    SPO => ['SPOt', 'R', '64', 'E', 0, 1],
    STA => ['STAsh', 'S', '10', 'C(R)', 0, 0],
    STO => ['STOre', 'S', '1', 'EA', 0, 0],
    STR => ['STRing', 'E', '97', 'C(N)', 0, 0],
    STS => ['STArt of STAtement', 'S', '0', '###C(#)S', 0, 0],
    STU => ['STUdy', 'S', '29', 'EER', 0, 0],
    SUB => ['SUBscript', 'R', '83', 'ER', 0, 1],
    SWA => ['SWAp', 'S', '37', 'OO', 0, 0],
    SWB => ['unary Subtract Without Borrow', 'E', '100', 'E', 0, 1],
    SYS => ['SYStem call', 'S', '43', 'EC(S)', 0, 0],
    TAI => ['TAIl', 'R', '66', 'E', 0, 1],
    TSP => ['Two SPot', 'R', '65', 'E', 0, 1],
    TYP => ['TYPe', 'R', '79', 'RE', 0, 1],
    UDV => ['Unary DiVide', 'E', '110', 'E', 0, 1],
    UNE => ['UNdocumented Expression', 'E', '44', 'EEC(E)', 0, 0],
    UNS => ['UNdocumented Statement', 'S', '44', 'EEC(E)', 0, 0],
    WHP => ['WHirlPool', 'R', '68', 'E', 0, 1],
    WIN => ['Write IN', 'S', '19', 'C(A)', 0, 0],
);

my %bytedecode = (
    0 => 'STS',
    1 => 'STO',
    2 => 'CRE',
    3 => 'DES',
    4 => 'MSP',
    5 => 'DSX',
    6 => 'NOT',
    7 => 'NXT',
    8 => 'RES',
    9 => 'FOR',
    10 => 'STA',
    11 => 'RET',
    12 => 'IGN',
    13 => 'REM',
    14 => 'ABL',
    15 => 'ABG',
    16 => 'REL',
    17 => 'REG',
    18 => 'GUP',
    19 => 'WIN',
    20 => 'ROU',
    21 => 'LAB',
    22 => 'CFL',
    23 => 'CFG',
    24 => 'QUA',
    25 => 'BWC',
    26 => 'CWB',
    27 => 'ENS',
    28 => 'FRE',
    29 => 'STU',
    30 => 'ENR',
    31 => 'LEA',
    32 => 'FIN',
    33 => 'GRA',
    34 => 'NXL',
    35 => 'NXG',
    36 => 'CON',
    37 => 'SWA',
    38 => 'BUG',
    39 => 'OPT',
    40 => 'EBC',
    41 => 'ECB',
    42 => 'FRZ',
    43 => 'SYS',
    44 => 'UNS',
    44 => 'UNE',
    44 => 'UNS',
    44 => 'UNE',
    64 => 'SPO',
    65 => 'TSP',
    66 => 'TAI',
    67 => 'HYB',
    68 => 'WHP',
    69 => 'DOS',
    70 => 'SHF',
    71 => 'CHO',
    79 => 'TYP',
    80 => 'OVR',
    81 => 'ROR',
    82 => 'OWN',
    83 => 'SUB',
    96 => 'MUL',
    97 => 'STR',
    98 => 'BUT',
    99 => 'BBT',
    100 => 'SWB',
    101 => 'BSW',
    102 => 'AWC',
    103 => 'BAW',
    104 => 'SEL',
    105 => 'INT',
    106 => 'NUM',
    107 => 'OVM',
    108 => 'ROM',
    109 => 'SPL',
    110 => 'UDV',
    126 => 'HSN',
    127 => 'OSN',
);

my @bc_list = qw(
    ABG ABL AWC BAW BBT BSW BUG BUT BWC CFG CFL CHO CON CRE CWB DES DOS DSX
    EBC ECB ENR ENS FIN FOR FRE FRZ GRA GUP HSN HYB IGN INT LAB LEA MSP MUL
    NOT NUM NXG NXL NXT OPT OSN OVM OVR OWN QUA REG REL REM RES RET ROM ROR
    ROU SEL SHF SPL SPO STA STO STR STS STU SUB SWA SWB SYS TAI TSP TYP UDV
    UNE UNS WHP WIN
);

sub BC_ABG () { 15; }
sub BC_ABL () { 14; }
sub BC_AWC () { 102; }
sub BC_BAW () { 103; }
sub BC_BBT () { 99; }
sub BC_BSW () { 101; }
sub BC_BUG () { 38; }
sub BC_BUT () { 98; }
sub BC_BWC () { 25; }
sub BC_CFG () { 23; }
sub BC_CFL () { 22; }
sub BC_CHO () { 71; }
sub BC_CON () { 36; }
sub BC_CRE () { 2; }
sub BC_CWB () { 26; }
sub BC_DES () { 3; }
sub BC_DOS () { 69; }
sub BC_DSX () { 5; }
sub BC_EBC () { 40; }
sub BC_ECB () { 41; }
sub BC_ENR () { 30; }
sub BC_ENS () { 27; }
sub BC_FIN () { 32; }
sub BC_FOR () { 9; }
sub BC_FRE () { 28; }
sub BC_FRZ () { 42; }
sub BC_GRA () { 33; }
sub BC_GUP () { 18; }
sub BC_HSN () { 126; }
sub BC_HYB () { 67; }
sub BC_IGN () { 12; }
sub BC_INT () { 105; }
sub BC_LAB () { 21; }
sub BC_LEA () { 31; }
sub BC_MSP () { 4; }
sub BC_MUL () { 96; }
sub BC_NOT () { 6; }
sub BC_NUM () { 106; }
sub BC_NXG () { 35; }
sub BC_NXL () { 34; }
sub BC_NXT () { 7; }
sub BC_OPT () { 39; }
sub BC_OSN () { 127; }
sub BC_OVM () { 107; }
sub BC_OVR () { 80; }
sub BC_OWN () { 82; }
sub BC_QUA () { 24; }
sub BC_REG () { 17; }
sub BC_REL () { 16; }
sub BC_REM () { 13; }
sub BC_RES () { 8; }
sub BC_RET () { 11; }
sub BC_ROM () { 108; }
sub BC_ROR () { 81; }
sub BC_ROU () { 20; }
sub BC_SEL () { 104; }
sub BC_SHF () { 70; }
sub BC_SPL () { 109; }
sub BC_SPO () { 64; }
sub BC_STA () { 10; }
sub BC_STO () { 1; }
sub BC_STR () { 97; }
sub BC_STS () { 0; }
sub BC_STU () { 29; }
sub BC_SUB () { 83; }
sub BC_SWA () { 37; }
sub BC_SWB () { 100; }
sub BC_SYS () { 43; }
sub BC_TAI () { 66; }
sub BC_TSP () { 65; }
sub BC_TYP () { 79; }
sub BC_UDV () { 110; }
sub BC_UNE () { 44; }
sub BC_UNS () { 44; }
sub BC_WHP () { 68; }
sub BC_WIN () { 19; }

my @reg_list = qw(
    AR AV AW BA CF CR CW DM ES EV FS IO IS JS OR ORFH OS OSFH OWFH PS RT
    SNFH SP SS TM TRFH WT
);

my %reg_list = (
    AR => ['spot', 0, BC_DOS, '%', 10],
    AW => ['spot', 0, BC_DOS, '%', 11],
    BA => ['base', 2, BC_DOS, '%', 4],
    CF => ['comefrom', 0, BC_DOS, '%', 5],
    CR => ['charset', 0, BC_DOS, '%', 6],
    CW => ['charset', 0, BC_DOS, '%', 7],
    DM => ['zeroone', 0, BC_DOS, '%', 18],
    ES => ['symbol', 'CALC_EXPR', BC_DOS, '%', 16],
    FS => ['symbol', 'CALC_FULL', BC_DOS, '%', 15],
    IO => ['iotype', 0, BC_DOS, '%', 3],
    IS => ['symbol', 0, BC_DOS, '%', 17],
    JS => ['symbol', 'END_JUNK', BC_DOS, '%', 12],
    OS => ['spot', 0, BC_DOS, '%', 8],
    PS => ['symbol', 'PROGRAM', BC_DOS, '%', 14],
    RT => ['roman', 0, BC_DOS, '%', 2],
    SP => ['splat', 1000, BC_DOS, '%', 19],
    SS => ['symbol', 'SPACE', BC_DOS, '%', 13],
    TM => ['zeroone', 0, BC_DOS, '%', 9],
    WT => ['zeroone', 0, BC_DOS, '%', 1],
    AV => ['vector', [], BC_SHF, '^', 1],
    EV => ['vector', [], BC_SHF, '^', 2],
    OR => ['whirlpool', undef, BC_WHP, '@', 0],
    ORFH => ['whirlpool', $stdread, BC_WHP, '@', 2],
    OSFH => ['whirlpool', $stdsplat, BC_WHP, '@', 3],
    OWFH => ['whirlpool', $stdwrite, BC_WHP, '@', 1],
    SNFH => ['whirlpool', $devnull, BC_WHP, '@', 7],
    TRFH => ['whirlpool', $stdsplat, BC_WHP, '@', 9],
);

my %reg_names = (
    '%1' => 'WT',
    '%2' => 'RT',
    '%3' => 'IO',
    '%4' => 'BA',
    '%5' => 'CF',
    '%6' => 'CR',
    '%7' => 'CW',
    '%8' => 'OS',
    '%9' => 'TM',
    '%10' => 'AR',
    '%11' => 'AW',
    '%12' => 'JS',
    '%13' => 'SS',
    '%14' => 'PS',
    '%15' => 'FS',
    '%16' => 'ES',
    '%17' => 'IS',
    '%18' => 'DM',
    '%19' => 'SP',
    '^1' => 'AV',
    '^2' => 'EV',
    '@0' => 'OR',
    '@1' => 'OWFH',
    '@2' => 'ORFH',
    '@3' => 'OSFH',
    '@7' => 'SNFH',
    '@9' => 'TRFH',
);

my %mulmap = map { ( $_ => 1 ) } BC_MUL, BC_STR;

sub bc_list () {
    @bc_list;
}

sub BC {
    @_ == 1 || croak "Usage: BC(value)";
    my ($val) = @_;
    croak "Invalid undefined value" unless defined $val;
    my $orig = $val;
    $val < BYTE_SHIFT
	and return ($val + NUM_OPCODES);
    $val < OPCODE_RANGE
	and return (BC_HSN, $val);
    my $div = int($val / OPCODE_RANGE);
    $div < OPCODE_RANGE
	and return (BC_OSN, $div, $val % OPCODE_RANGE);
    croak "Invalid value $orig: does not fit in one spot";
}

sub bytecode ($) {
    my ($name) = @_;
    $name =~ /^\d+$/ && $name < BYTE_SHIFT ? ($name + NUM_OPCODES)
					   : $bytecodes{$name}[2];
}

sub bytedecode ($) {
    my ($b) = @_;
    if ($b >= NUM_OPCODES) {
	my $n = $b - NUM_OPCODES;
	return () if $n >= BYTE_SHIFT;
	return "#$n" unless wantarray;
	return ("#$n", 'Constant', '#', $b, '', 1, 1);
    } else {
	return () unless exists $bytedecode{$b};
	return $bytedecode{$b} unless wantarray;
	return ($bytedecode{$b}, @{$bytecodes{$bytedecode{$b}}});
    }
}

sub BCget {
    @_ == 3 or croak "Usage: BCget(CODE, \\POSITION, END)";
    my ($code, $cp, $ep) = @_;
    $$cp >= $ep and faint(SP_INVALID, "end of code", "BCget");
    my $byte = ord(substr($code, $$cp, 1));
    $$cp++;
    if ($byte >= NUM_OPCODES) {
	return $byte - NUM_OPCODES;
    }
    if ($byte == BC_HSN) {
confess "HSN $$cp >= $ep" if $$cp >= $ep;
	$$cp >= $ep and faint(SP_INVALID, "end of code", "BCget/HSN");
	return ord(substr($code, $$cp++, 1));
    }
    if ($byte == BC_OSN) {
confess "OSN $$cp + 1 >= $ep" if $$cp + 1 >= $ep;
	$$cp + 1 >= $ep and faint(SP_INVALID, "end of code", "BCget/OSN");
	my $nx = unpack('n', substr($code, $$cp, 2));
	$$cp += 2;
	return $nx;
    }
confess sprintf("Invalid bytecode 0x%02X", $byte); # XXX
    faint(SP_INVALID, sprintf("0x%02x", $byte), "BCget")
}

sub BC_constants () {
    (NUM_OPCODES..BC_MASK);
}

sub is_constant ($) {
    my ($byte) = @_;
    return 1 if $byte >= NUM_OPCODES ||
		$byte == BC_HSN ||
		$byte == BC_OSN;
    return 0;
}

sub is_multibyte ($) {
    my ($byte) = @_;
    return 1 if $byte == BC_HSN;
    return 2 if $byte == BC_OSN;
    0;
}

sub bc_bytype {
    @_ or croak "Usage: bc_bytype(TYPES)";
    my %types = ();
    for my $type (@_) {
	if ($type eq 'R' || $type eq 'S') {
	    $types{$type} = 0;
	    next;
	}
	if ($type =~ /^[CEP<>L\[\]]$/) {
	    $types{E} = $types{R} = $types{'#'} = 0;
	    next;
	}
	if ($type eq 'V') {
	    $types{R} = $types{V} = 0;
	    next;
	}
	if ($type eq 'O') {
	    $types{S} = 0;
	    next;
	}
    }
    my %values = exists $types{V} ? %mulmap : ();
    map {
	my ($desc, $type, $value, $args, $function) = @{$bytecodes{$_}};
	if (exists $types{$type} || exists $values{$value}) {
	    $value;
	} else {
	    ();
	}
    } keys %bytecodes;
}

sub bc_match {
    @_ >= 2 && @_ <= 4
	or croak "Usage: bc_match(PATTERN, CODE [,START [,END]])";
    my ($pattern, $code, $start, $end) = @_;
    $start ||= 0;
    $end = length($code) if not defined $end;
    _match($pattern, $code, $start, $end, undef);
}

sub bc_skip {
    @_ >= 1 && @_ <= 3
	or croak "Usage: bc_skip(CODE [,START [,END]])";
    my ($code, $start, $end) = @_;
    $start ||= 0;
    $end = length($code) if not defined $end;
    return undef if $start >= $end || $start < 0;
    my $byte = ord(substr($code, $start, 1));
    return 1 if $byte >= NUM_OPCODES;
    return undef if ! exists $bytedecode{$byte};
    my $name = $bytedecode{$byte};
    my $pattern = $bytecodes{$name}[1];
    _match($pattern, $code, $start, $end, undef);
}

sub bc_forall {
    @_ == 5
	or croak "Usage: bc_forall(PATTERN, CODE, START, END, CLOSURE)";
    my ($pattern, $code, $start, $end, $closure) = @_;
    $start ||= 0;
    $end = length($code) if not defined $end;
    return undef if $start >= $end || $start < 0;
    my $np = '';
    while ($pattern =~ s/^(.*?)C\(/(/) {
	my $a = $1;
	$a =~ s/(.)/$1\x01/g;
	$np .= $a . 'C';
	$np .= '(' . _args('forall', \$pattern) . ')';
	$np .= "\01";
    }
    $pattern =~ s/(.)/$1\x01/g;
    $pattern = "\x01" if $pattern eq '';
    $np .= $pattern;
    _match($np, $code, $start, $end, $closure);
}

sub bc_xtype {
    @_ == 1 or croak "Usage: bc_xtype(\\PATTERN)";
    my ($pattern) = @_;
    _args('xtype', $pattern);
}

my %typemap = (
    'S' => { 'S' => 0 },
    'O' => { 'S' => 0 },
    'E' => { 'E' => 0, 'R' => 0, '#' => 0 },
    'A' => { 'E' => 0, 'R' => 0, '#' => 0 },
    'R' => { 'R' => 0 },
    'V' => { 'R' => 0, 'V' => 0 },
    '#' => { '#' => 0 },
    'C' => { '#' => 0 },
    'Z' => { 'S' => 0, 'E' => 0, 'R' => 0, '#' => 0 },
    '*' => { 'S' => 0, 'E' => 0, 'R' => 0, '#' => 0 },
);

sub _args {
    my ($name, $pattern) = @_;
    faint(SP_BCMATCH, $name, 'Missing (') if $$pattern !~ s/^\(//;
    my $count = 1;
    my $result = '';
    while ($count > 0) {
	$$pattern =~ s/^([^\(\)]*)([\(\)])//
	    or faint(SP_BCMATCH, $name, 'Missing )');
	$count++ if $2 eq '(';
	$count-- if $2 eq ')';
	$result .= $1 . ($count ? $2 : '');
    }
    $result;
}

sub _match {
    my ($pattern, $code, $sc, $ep, $closure) = @_;
    my $osc = $sc;
    MATCH: while ($pattern ne '') {
	my $e = substr($pattern, 0, 1, '');
	if ($e eq "\x00") {
	    $closure->(undef, '>') if $closure;
	    next MATCH;
	}
	if ($e eq "\x01") {
	    $closure->($sc, undef) if $closure;
	    next MATCH;
	}
if ($sc >= $ep) { $pattern =~ s/\x00/\\x00/g; confess "_match/0 $sc >= $ep ($pattern)" }
	faint(SP_INVALID, 'end of code', '_match') if $sc >= $ep;
	my $v = ord(substr($code, $sc, 1));
	if (exists $typemap{$e}) {
	    # check next opcode is correct type
	    my ($op, $desc, $type, $value, $args, $function) = bytedecode($v);
confess "_match/1 $v $e $pattern" unless defined $type;
	    faint(SP_INVALID, $v, "_match: $e")
		unless defined $type;
confess "_match/2 $v($op) $e $type $pattern (sc=$sc)" unless exists $typemap{$e}{$type} || (exists $mulmap{$v} && exists $typemap{$e}{V});
	    faint(SP_INVALID, $type, "_match: $e")
		unless exists $typemap{$e}{$type} ||
		       (exists $mulmap{$v} && exists $typemap{$e}{V});
	    if ($type eq '#' && $e ne '*') {
		my $num = BCget($code, \$sc, $ep);
		$closure->($v, "#$num") if $closure;
		if ($e eq 'C') {
		    $args = _args('count', \$pattern) x $num;
		    $args .= "\x00";
		    $closure->(undef, '<') if $closure;
		} else {
		    $args = '';
		}
	    } else {
		$sc++;
		$args = '' if $e eq 'O' || $e eq '*';
		$closure->($v, $op) if $closure;
	    }
	    $pattern = $args . $pattern;
	    next MATCH;
	} elsif ($e eq 'N') {
	    # any nonzero number
	    return undef if $v == 0;
	    $closure->($v, "N$v") if $closure;
	    $sc++;
	} elsif ($e eq '<') {
	    # left grammar element
	    my $count = BCget($code, \$sc, $ep);
	    my $num = BCget($code, \$sc, $ep);
	    if ($num == 0) {
		$closure->(undef, '?<') if $closure;
	    } elsif ($num == 1 || $num == 2) {
		$closure->(undef, ',<') if $closure;
	    } else {
		$closure->(undef, ',!<') if $closure;
	    }
	    if ($count && $closure) {
		$closure->(undef, $count == 65535 ? '*' : $count);
	    }
	    $pattern = "E\x00" . $pattern;
	    next MATCH;
	} elsif ($e eq '>') {
	    # right grammar element
	    my $num = BCget($code, \$sc, $ep);
	    if ($num == 0 || $num == 6) {
		my $count = BCget($code, \$sc, $ep);
		if ($count && $closure) {
		    $closure->(undef, $count);
		}
		$closure->($v, $num ? '!<' : '?<') if $closure;
		$pattern = "E\x00" . $pattern;
		next MATCH;
	    }
	    if ($num == 1 || $num == 2) {
		$closure->($v, ',<') if $closure;
		my $count = BCget($code, \$sc, $ep);
		if ($count && $closure) {
		    $closure->(undef, $count);
		}
		$pattern = "E\x00" . $pattern;
		next MATCH;
	    }
	    if ($num == 3 || $num == 7) {
		$closure->($v, ',!<') if $closure;
		my $count = BCget($code, \$sc, $ep);
		if ($count && $closure) {
		    $closure->(undef, $count);
		}
		$pattern = "E\x00" . $pattern;
		next MATCH;
	    }
	    if ($num == 4) {
		$num = BCget($code, \$sc, $ep);
		my $se = $sc + $num;
		$se <= $ep
		    or faint(SP_INVALID, '???', '_match: >');
		if ($closure) {
		    $closure->(undef, '=<');
		    while ($sc < $se) {
			$sc += _match('*', $code, $sc, $se, $closure);
		    }
		    $closure->(undef, '>');
		} else {
		    $sc = $se;
		}
		next MATCH;
	    }
	    if ($num == 15) {
		$closure->($v, '*') if $closure;
		next MATCH;
	    }
	    faint(SP_INVALID, $num, "_match: >");
	} elsif ($e eq '[') {
	    # XXX left optimise element
	    faint(SP_TODO, 'match on [');
	} elsif ($e eq ']') {
	    # XXX right optimise element
	    faint(SP_TODO, 'match on ]');
	} else {
	    faint(SP_BCMATCH, 'type', $e);
	}
    }
    $sc - $osc;
}

sub reg_list () {
    @reg_list;
}

sub reg_create {
    @_ == 2 || @_ == 3
	or croak "Usage: reg_create(REGISTER, OBJECT [, VALUE])";
    my ($rn, $object, @value) = @_;
    $rn = $reg_names{$rn} if exists $reg_names{$rn};
    if (exists $reg_list{$rn}) {
	@value = $reg_list{$rn}[1] if ! @value;
	my $rt = $reg_list{$rn}[3];
	my $dt = $reg_list{$rn}[0];
	return Language::INTERCAL::DoubleOhSeven->new($dt, $object, @value)
	    if $rt eq '%';
	return Language::INTERCAL::SharkFin->new($dt, $object, @value)
	    if $rt eq '^';
	return Language::INTERCAL::Whirlpool->new(@value)
	    if $rt eq '@';
    }
    $rn =~ /^\./
	and return Language::INTERCAL::Numbers::Spot->new(@value || 0);
    $rn =~ /^:/
	and return Language::INTERCAL::Numbers::Twospot->new(@value || 0);
    $rn =~ /^,/
	and return Language::INTERCAL::Arrays::Tail->new(@value || []);
    $rn =~ /^;/
	and return Language::INTERCAL::Arrays::Hybrid->new(@value || []);
    $rn =~ /^\@/
	and return Language::INTERCAL::Whirlpool->new();
    faint(SP_SPECIAL, $rn);
}

sub reg_codetype {
    @_ == 1 or croak "Usage: reg_codetype(REGISTER)";
    my ($rn) = @_;
    exists $reg_list{$rn} and return $reg_list{$rn}[0];
    if (exists $reg_names{$rn}) {
	$rn = $reg_names{$rn};
	return $reg_list{$rn}[0];
    }
    $rn =~ /^\./ and return 'spot';
    $rn =~ /^:/ and return 'twospot';
    $rn =~ /^,/ and return 'tail';
    $rn =~ /^;/ and return 'hybrid';
    $rn =~ /^\@/ and return 'whirlpool';
    faint(SP_SPECIAL, $rn);
}

sub reg_name {
    @_ == 1 or croak "Usage: reg_name(REGISTER)";
    my ($rn) = @_;
    exists $reg_list{$rn}
	and return $reg_list{$rn}[3] . $reg_list{$rn}[4];
    if (exists $reg_names{$rn}) {
	$rn = $reg_names{$rn};
	return $reg_list{$rn}[3] . $reg_list{$rn}[4];
    }
    $rn =~ /^([%^\@])(.*)$/ && exists $reg_list{$2} && $reg_list{$2}[3] eq $1
	and return $reg_list{$2}[3] . $reg_list{$2}[4];
    $rn =~ s/^([\.:,;\@^%])0*(\d+)$/$1$2/ and return $rn;
    undef;
}

sub reg_code {
    @_ == 1 or croak "Usage: reg_code(REGISTER)";
    my ($rn) = @_;
    exists $reg_list{$rn}
	and return ($reg_list{$rn}[2], BC($reg_list{$rn}[4]));
    if (exists $reg_names{$rn}) {
	$rn = $reg_names{$rn};
	return ($reg_list{$rn}[2], BC($reg_list{$rn}[4]));
    }
    $rn =~ /^([%^\@])(.*)$/ && exists $reg_list{$2} && $reg_list{$2}[3] eq $1
	and return ($reg_list{$2}[2], BC($reg_list{$2}[4]));
    $rn =~ /^\.(\d+)$/ and return (BC_SPO, BC($1));
    $rn =~ /^:(\d+)$/ and return (BC_TSP, BC($1));
    $rn =~ /^,(\d+)$/ and return (BC_TAI, BC($1));
    $rn =~ /^;(\d+)$/ and return (BC_HYB, BC($1));
    $rn =~ /^\@(\d+)$/ and return (BC_WHP, BC($1));
    $rn =~ /^\%(\d+)$/ and return (BC_DOS, BC($1));
    $rn =~ /^\^(\d+)$/ and return (BC_SHF, BC($1));
    undef;
}

sub reg_decode {
    @_ == 1 or croak "Usage: reg_name(REGISTER)";
    my ($rn) = @_;
    return $rn if $rn =~ /^[.,:;\@_]/;
    if ($rn =~ /^[%^]\d+$/) {
	return undef unless exists $reg_names{$rn};
	$rn = $reg_names{$rn};
    } elsif ($rn =~ s/^([%^])//) {
	return undef unless exists $reg_list{$rn};
	return undef if $1 ne $reg_list{$rn}[3];
    } else {
	return undef unless exists $reg_list{$rn};
    }
    $reg_list{$rn}[3] . $rn;
}

1;

__END__

=pod

=head1 TITLE

Language::INTERCAL::Bytecode - definitions for the ICBM

=head1 DESCRIPTION

The ICBM (B<I>NTERCAL B<C>ommon B<B>ytecode B<M>angler) is the part of
CLC-INTERCAL which defines the INTERCAL virtual machine, that is the
execution environment for interpreted INTERCAL programs, as well as the
intermediate language used to compile programs.

The compiler produces bytecode from program source; this bytecode can
be interpreted to execute the program immediately; alternatively, a
backend can produce something else from the bytecode, for example C
or Perl source code which can then be compiled to your computer's
native object format.

The compiler itself is just some more bytecode. Thus, to produce the
compiler you need a compiler compiler, and to produce that you need
a compiler compiler compiler; to produce the latter you would need
a compiler compiler compiler compiler, and so on to infinity. To
simplify the programmer's life (eh?), the compiler compiler is able
to compile itself, and is therefore identical to the compiler compiler
compiler (etcetera).

The programmer can start the process because a pre-compiled compiler
compiler, in the form of bytecode, is provided with the CLC-INTERCAL
distribution; this compiler compiler then is able to compile all
other compilers, as well as to rebuild itself if need be.

This document describes the bytecode used by all user programs, compilers,
and the compiler compiler. Other documentation, if and when it will be
written, describes the compilation process in more detail as well as the
rest of the execution environment.

A program, in its executable form, consists of a list of statements. Each
statement can contain alternative execution paths. Execution starts at
the first statement in the list and proceed in sequence unless the bytecode
has some form of jump, next or is subject to a come from. The choice of a
particular execution path amongst the various possibilities will be discussed
later.

=head1 BYTECODE DEFINITIONS

Each byte in the bytecode can be part of a statement, an expression or a
register. In addition, a subset of expressions can be assigned to: these
are called assignable expressions. For example, a constant is an assignable
expression. When assigned to, it changes the value of the constant. This
is necessary to implement overloading and is also a great obfuscation
mechanism.

The rest of this document uses the three-letter abbreviations for bytes in
the bytecode. To see the corresponding numeric code, use:

    perl -lMLanguage::INTERCAL::ByteCode=:BC -e 'print BC_???'

where I<???> is the three letter abbreviation used in the text. To see the
one-line description type:

    perl -lMLanguage::INTERCAL::ByteCode=:BC,bytedecode -e \
    'print ((bytedecode(BC_???))[1])'

If you see the number anywhere and want to have the three-letter abbreviation
type:

    perl -lMLanguage::INTERCAL::ByteCode=:bytedecode -e \
    'print scalar bytedecode(nnn)'

Your shell or command interpreter might have its own idea about the above
commands and the use of quotes, as usual.

=head2 CONSTANTS

Constants can be specified in four ways:

=over 4

=item ###

Byte larger than maximum opcode.

Any byte with value greather than the maximum opcode is interpreted as a
16 bit (spot) constant by subtracting the number of opcodes from the byte.
For example, since there are 128 opcodes, byte 131 is equivalent to #3,
and byte 255 (the maximum value) is #127

=item HSN

Half Spot Number

Followed by a second byte, represents the value of that byte.

=item OSN

One Spot Number

Followed by two bytes, represents the 16 bit number which has the first such
byte as higher significant half, and the second byte as lower significant
half.

=back

The exportable function I<BC> converts a number into bytecode, returning
a list of numbers: use C<pack('C*', BC($n)> to convert a number to a
bytecode string.

The exportable function I<BCget> converts a bytecode string to a number.
The function takes three parameters: the string containing the bytecode,
a scalar reference indicating the starting point (this will be updated
to the first opcode after the constant) and the end of the bytecode
block one wants to examine.

As the number of opcodes can change, it is recommended to use I<BC> and
I<BCget> whenever possible to ensure compatibility with future versions
of CLC-INTERCAL.

=head2 REGISTERS

Registers can be any number of register prefixes, followed by a type and
a constant. There are limitations in the useful combination of prefixes.

The register types are:

=over 4

=item CHO

Crawling HOrror

Crawling horror: a special register holding a compiler, grammar or something
similar. Currently, these registers cannot be used directly, they are
implicitely used by I<CRE> and I<DES>.

=item DOS

Double-Oh-Seven

Double-oh-seven: a special internal spot register used by the compilers.

=item HYB

HYBrid

Hybrid register (e.g. I<;9>). This represents the whole array. See I<SUB>
for subscripting.

=item SHF

SHark Fin

Shark fin: a special internal tail register used by the compilers (e.g.
I<^1>, the arguments given to the program on startup)

=item SPO

SPOt

Spot register (e.g. I<.4>)

=item TAI

TAIl

Tail register (e.g. I<,2>). This represents the whole array. See I<SUB> for
subscripting.

=item TSP

Two SPot

Two spot register (e.g. I<:7>)

=item TYP

TYPe

Followed by any register, returns its type. For example, I<TYP> I<SPO>
I<136> is equivalent to I<SPO>. It can be useful to find out the type of an
indirect register, and is used to translate CLC-INTERCAL's
intersection-worm.

=item WHP

WHirlPool

Whirlpool (e.g. I<@9>). This represents CLC-INTERCAL's class registers. When
used for I/O, it represents the filehandle associated with the class.

=back

Crawling horror (_) registers are not explicitely specified; rather, the
operations which take a crawling horror accept the register number and
form the complete register name internally.

The prefixes which can applied to registers are:

=over 4

=item OVR

OVerload Register

Followed by an expression and a register, overloads the register and returns
the register itself.

=item OWN

OWNer

Followed by a constant and a register, takes the corresponding owner from
the register.

=item ROR

Remove Overload Register

Followed by a register name, removes any overloads and returns the register
itself. Used by the optimiser. Assignable.

=item SUB

SUBscript

Followed by an expression and a subscriptable register, it accesses the
given subscript. For multidimensional arrays, repeat as in I<SUB> I<67>
I<SUB> I<68> I<TAI> I<69> for I<:5 SUB #4 SUB #3>. In addition to hybrid,
tail and shark fin registers, whirlpools also accept subscripts, allowing to
access the subjects directly.

=back

=head2 ASSIGNABLE EXPRESSIONS

Assignable expressions are sequences of bytecode which can used as the
target of an assignment. Of course, all registers are assignable;
all constants are also assignable, which makes then really variables.
Instead of describing the assignable expressions separately, we describe
all expressions and mention which ones are assignable. Assigning to
an expression means assigning appropriate values to its subexpressions such
that the expression, if evaluated, would result in the value being assigned.
This is not always possible, so it can generate runtime errors.

=head2 EXPRESSIONS

In addition to registers and constants, the following are valid expressions:

=over 4

=item AWC

unary Add Without Carry

Followed by one expression, it computes the unary Add without carry; invalid
in base 2. Assignable if the argument is.

=item BAW

binary Add Without Carry

Binary version of I<AWC>. Used by the optimiser. Assignable if its arguments
are.

=item BBT

binary BUT

Binary version of I<BUT>. Used by the optimiser. Assignable if its arguments
are.

=item BSW

binary Subtract Without Borrow

Binary version of I<SWB>. Used by the optimiser. Assignable if its arguments
are.

=item BUT

unary BUT

Followed by two expressions, computes the unary I<BUT> of the second
expression, preferring the value of the first - so this can also be used for
unary I<3BUT> etc. The special prevference value 7, which is invalid for
unary I<BUT>, is used to indicate unary I<AND>. Assignable if the second
argument is assignable.

=item INT

INTerleave

Followed by two expressions, interleaves them. Assignable if both arguments
are assignable.

=item MUL

MULtiple number

Followed by an expression (the count) and then a number of expressions,
represents a ``multiple number''. This can be used to assign to an array, to
dimension it (e.g. translating the statement C<,2 <- #3 BY #5> the value to
be assigned would be I<MUL> I<66> I<67> I<69>). Not assignable.

=item NUM

NUMber

Followed by a register, returns its number. So for example I<NUM> I<SPO>
I<#2> would be the same as I<#2>. This is more useful when the register
provided is reached using I<OWN>. Assignable.

=item OVM

OVerload Many

Followed by two expressions, overloads a range of registers. Note that all
types of registers are overloaded. The range is determined by uninterleaving
the second argument. See also I<OVR>. Assignable.

=item ROM

Remove Overload Many

Followed by an expression, removes overload from a range of of registers.
Used by the optimiser. Assignable.

=item SEL

SELect

Followed by two expressions, it selects them. Assignable if the arguments
are assignable.

=item SPL

SPLat

Returns the code of the last splat. This is only useful if the program is
quantum or threaded, otherwise it won't be executing after a splat. If there
was no splat, generates one. Assignable, but in this case it unconditionally
splats for obvious reasons.

=item STR

STRing

Similar to I<MUL>, but used in the special case where all the arguments are
constant characters. This may result in internal optimisations and the like.
Otherwise it is just a more compact way of using I<MUL> where all arguments
are constants and fit in a byte. Not assignable.

=item SWB

unary Subtract Without Borrow

Followed by one expression, it computes the unary subtract without borrow.
In base 2, corresponds to the unary exclusive or. Assignable if the argument
is.

=item UDV

Unary DiVide

The "most useless" operation, but surely somebody will find a use for it.
This operation can be considered arithmetic or bitwise, depending on the
value of special register I<%DM>.

=item UNE

UNdocumented Expression

Can't really tell you what this does, can we?

=back

=head2 STATEMENTS

The following opcodes are valid statements:

=over 4

=item ABG

ABstain from Gerund

Followed by a constant (the count) and I<count> gerunds, ABSTAINs FROM the
corresponding statement(s).

=item ABL

ABstain from Label

Followed by an expression, representing a label, ABSTAINs FROM the
corresponding statement(s).

=item BUG

compiler BUG

This opcode is automatically inserted by the compiler where appropriate.
Takes one argument, the bug type (#0 - explainable, #1 - unexplainable).

=item BWC

loop: Body While Condition

Followed by two statements, executes a loop. This implements the (non
default) loop where the body is before the WHILE and the condition after.
The first statement is the condition and the second is the body, not the
other way around as one would expect.

=item CFG

Come From Gerund

Followed by a constant (the count) and I<count> gerunds (opcodes), executes
a COME FROM gerund. The special register I<%CF> determines, amongst other
things, whether these statements are really executed or not. The default is
not, and linking a program with the object I<come-from-gerund.io> will
change this register to allow these statements: this object is normally
linked automatically when the program source has a suffix I<.gi>. See I<CFL>
for other functions of the I<%CF> register.

=item CFL

Come From Label

Followed by an expression, executes a COME FROM label. The special register
I<%CF> determines, amongst other things, whether it is admissible to have
multiple COME FROM (and NEXT FROM) all pointing at the same label. The
default is to cause a splat; linking with the object I<thick.io> changes
this to allow multiple COME FROMs and NEXT FROMs to create threads.

=item CON

CONvert

Followed by two opcodes, converts the first into the second. The two opcodes
must be compatible, in the sense that they take the same operands.

=item CRE

CREate

Followed by two expressions (a grammar and a symbol), a constant (a left
count), I<left count> rules, another constant (a right count) and I<right
count> chunks of code, executes a CREATE statement. This is not documented
here but the compilers provide a large number of examples.

=item CWB

loop: Condition While Body

Followed by two statements, executes a loop. This implements the (default)
loop where the condition is before the WHILE and the body after. The first
statement is the body and the second is the condition, not the other way
around as one would expect.

=item DES

DEStroy

Followed by two expressions (a grammar and a symbol) and a constant (a left
count) and I<left count> rules, executes a DESTROY statement.

=item DSX

Double-oh-Seven eXecution

Followed by an expression (which should have value between 0 and 100), it
executes the statement with a probability indicated by the expression,
between 0% (never) and 100% (always).

=item EBC

Event: Body while Condition

Followed by an expression and a statement, schedules an event. This
implements the (non default) event where the body is before the WHILE and
the condition after, and therefore produces a runtime error unless its
implementation is CONVERTed to or SWAPped with I<ECB>.

=item ECB

Event: Condition while Body

Followed by an expression and a statement, schedules an event. This
implements the (default) event where the condition is before the WHILE and
the body after.

=item ENR

ENRol

Followed by a count of subjects, I<count> expressions representing the
subjects, and a register wishing to study these subjects, looks for a class
teaching the subjects and enrols the register there.

=item ENS

ENSlave

Followed by two registers, enslaves the first to the second.

=item FIN

FINish lecture

Execution continues after the I<LEA> which took us to the lecture. Also
frees the class register from the student.

=item FOR

FORget

Followed by an expression, pops that many levels from the stash containing
the return addresses for I<NXT>, I<NXL> and I<NXG> and throws these
addresses in the bit bucket.

=item FRE

FREe

Followed by two registers, frees the first from the second. It is an error
to free a register from another register it was not enslaved to.

=item FRZ

FReeZe

Freezes the current program by removing the source code and replacing the
grammar used to compile it with the "secondary" grammar; this means that
subsequent I<CRE> and I<DES> will be an error if they cause recompilation. A
compiler works by creating a secondary grammar, then freezing itself and
then continuing with the user's program, which is compiled using the new
grammar just created, rather than the one used to compile the compiler
itself.

=item GRA

GRAduate

Followed by a register (a student) it causes the student to graduate, that
is to drop all classes.

=item GUP

Give UP

Causes program termination. When used in a compiler module, causes module
processing to stop, the compiler will then load the next module or, if no
more modules are to be loaded, it will start compiling the program.

=item IGN

IGNore

Followed by a constant (a count) and I<count> registers, ignores the
registers.

=item LAB

LABel

Followed by an expression, indicates this statement's label. If the
expression is nonzero, after this statement the ICBM will go looking for
corresponding I<CFL> and I<NFL> statements (COME FROMs and NEXT FROMs). It
is also used to abstain/reinstate by label.

=item LEA

LEArns

Followed by an expression (the subject), and a register (the student) looks
for a lecture where that subject is taught in one of the classes the student
is enrolled in. The class register is temporarily enslaved to the student
and execution continues at the start of the lecture.

=item MSP

Make SPlat

Followed by an expression, the splat code, a number, the count, and I<count>
more expressions, generates a splat. The first expression is the splat code,
the rest are used to generate the splat message. The splat code determines
the correct number of arguments, if the wrong number is provided the message
may look weird.

=item NOT

NOT

Signals that this statement is initially abstained from. A statement might
be abstained from without containing a I<NOT>, or might contain one and not
be abstained from, depending on the I<ABL>, I<ABG>, I<REL> and I<REG>
executed since the start of the program.

=item NXG

Next From Gerund

Similar to I<CFG>, but executes a NEXT FROM instead: the difference is that
I<NXG> stashes the return address in the same way as I<NXT> does. See I<CFG>
and I<NXT>.

=item NXL

Next From Label

Similar to I<CFL>, but executes a NEXT FROM instead: the difference is that
I<NXL> stashes the return address in the same way as I<NXT> does. See I<CFL>
and I<NXT>.

=item NXT

NeXT

Followed by an expression, a label, stashes the address of the next
statement and continues execution at that label. It is an error if the label
is multiply defined or not defined at all.

=item OPT

OPTimise

Takes a constant (the left count), I<left count> patterns, a second constant
(the right count) and I<right count> replacements. Inserts the resulting
rule into the optimiser, which can do whatever it likes with it.

=item QUA

QUAntum statement

Executes the rest of the statement in "quantum bit creation" mode. This
means that anything which modifies data will end up creating quantum bits.

=item REG

REinstate from Gerund

Followed by a constant (the count) and I<count> gerunds, REINSTATEs the
corresponding statement(s).

=item REL

REinstate from Label

Followed by an expression, representing a label, REINSTATEs the
corresponding statement(s).

=item REM

REMember

Followed by a constant (a count) and I<count> registers, remembers the
registers.

=item RES

RESume

Followed by an expression, pops that many levels from the stash containing
the return addresses for I<NXT>, I<NXL> and I<NXG>. All the addresses except
one are then discarded, and execution continues at the last address
extracted from the stash.

=item RET

RETrieve

Followed by a constant (the count) and I<count> registers, RETRIEVEs these
registers.

=item ROU

Read OUt

Followed by a constant (a count) and I<count> expressions, reads them out.

=item STA

STAsh

Followed by a constant (the count) and I<count> registers, STASHes these
registers.

=item STO

STOre

Followed by two expressions, assigns the value of the first expression to
the second. It is common to have a register as the second expression, but
any assignable expression will do.

=item STS

STArt of STAtement

Takes a variable number of constants, not less than four. The first constant
indicates the byte position in the source code where this statement was
compiled from; the second constant indicates the length of the statement in
the source code; the third indicates whether the statement may be a comment
(it has not been recognised using the currently active grammar) or not; the
fourth indicates the number of constants following. The rest of the
constants indicate which grammar rules were used to compile this particular
statement.

At runtime, not all grammar rules may be available at all times, depending
on the history of I<CRE> and I<DES>. To execute a statement corresponding to
a given bit of source code, the runtime will find all relevant I<STS>
statements find the best one which could have been compiled given the
current state of the grammar, and executes it; if a non-comment statement is
available, it will be used, otherwise a comment one will have to do. If
execution is to proceed sequentially, the second constant is used to figure
out how to repeat this process. Execution starts at byte offset 0 in the
source code.

Grammar rules are numbered at compile time, and may differ from program to
program.

=item STU

STUdy

Followed by an expression (the subject), a label (the lecture) and a
register (the class), executes a STUDY statement.

=item SWA

SWAp

Followed by two opcodes, swaps them. The two opcodes must be compatible, in
the sense that they take the same operands.

=item SYS

SYStem call

Followed by an expression (the system call number), a count, and I<count>
statements (the system call implementation), defines a system call.

=item UNS

UNdocumented Statement

Can't really tell you what this does, can we?

=item WIN

Write IN

Followed by a constant (a count) and I<count> assignable expressions, writes
them in.

=back

=head1 SPECIAL REGISTERS

Bytecode operation is controlled by a number of special registers;
registers prefixed by a double-oh-seven (%) are special spot registers;
registers prefixed by a shark fin (^) are special tail registers;
whirlpool (@) registers are normal class registers, but also contain
special values;
these registers cannot be accessed directly by user programs (unless
one adds the corresponding syntax to the I<sick.io> compiler), but
are necessary for writing compilers, and are accessible to programs
written in assembler.

The double-oh-seven and shark fin registers are often referred to by
symbolic name, rather than number. The assembler recognises such names
and maps them to numbers. For this reason, the listing below uses the
names. Since trace outputs may show numbers rather than names, the
register number is also shown in parentheses.

=head2 DOUBLE-OH-SEVEN REGISTERS

Double-oh-seven registers hold one-spot numbers which can be useful for
compilers and interpreters.

=over 4

=item %AR

Array read value (%10)

Contains the last byte READ OUT when %IO is C.

=item %AW

Array write value (%11)

Contains the last byte WRITtEn IN when %IO is C.

=item %BA

BAse (%4)

Holds the base used for all arithmetic. Assigning a value less than 2 or
greater than 7 causes an error.

=item %CF

Come From style (%5)

This register can only hold values from #0 to #3. The lowest bit (in base 2)
determines what happens when multiple COME FROM or NEXT FROM all point at
the same label: if zero, you get a splat, if one you get a multithreaded
program. The other bit determines whether COME FROM gerund (and NEXT FROM
gerund) will work: zero disables these statements, one enables them. Thus
all compilers set %CF to #0, except I<thick.io> which sets it to #1 and
I<come-from-gerund.io> which sets it to #2.

=item %CR

Charset for Reads (%6)

The character set used by alphanumeric READ OUT when %IO is CLC. Assigning a
number to this register causes the corresponding style to be selected;
assigning a I<MUL> causes a symbolic lookup to determine the style number.
See L<Language::INTERCAL::Charset>.

=item %CW

Charset for Writes (%7)

The character set used by alphanumeric WRITE IN when %IO is CLC. Assigning a
number to this register causes the corresponding style to be selected;
assigning a I<MUL> causes a symbolic lookup to determine the style number.
See L<Language::INTERCAL::Charset>.

=item %DM

unary Division mode (%18)

This register can only have value #0 or #1, and determines the style of
unary division employed. The default value #0 corresponds to the
"arithmetic" style of division, while value #1 corresponds to the "bitwise"
style. See the I<UDV> opcode.

=item %ES

"expr" symbol (%16)

Used by the Intercal calculator (intercalc) to determine how to parse lines
when operating in "expr" mode.

=item %FS

"full" symbol (%15)

Used by the Intercal calculator (intercalc) to determine how to parse lines
when operating in "full" mode.

=item %IO

I/O type (%3)

Determines how non-numeric WRITE IN and READ OUT work. Assigning a number to
this register causes the corresponding style to be selected; assigning a
I<MUL> causes a symbolic lookup to determine the style number. See
L<Language::INTERCAL::ArrayIO>.

=item %IS

Intersection symbol (%17)

Determines what separates statements in the program; this is not currently
used in any compiler and can be left at the default, zero. If set to any
other value, the corresponding grammar symbol is used to compile the bit of
source between consecutive statements; if this generates code, it will be
executed with the preceding statement. Changing the value of this register
at runtime can be a great obfuscation tool. See also I<%PS>.

=item %JS

Junk Symbol (%12)

When parsing a comment, the compiler needs to be told how to recognise the
start of next statement: for example, 1972.io and ick.io set this to a
grammar symbol meaning "optional (number) followed by DO or PLEASE"; sick.io
does something similar, but the complication caused by computed labels (if
enabled) makes it alightly more difficult to describe what this symbol does.

Changing the value of this register at runtime can be a great obfuscation
tool.

=item %OS

Operating System (%8)

Hidden in the darkest corner of the operating system lurks a "DO NEXT FROM
%OS". As long as %OS has the default value of zero, you are safe from this.

If %OS is assigned some other value, it behaves like a normal (?) NEXT FROM,
with one added twist to do with parameter passing. Every time your program
assigns a value to a register, %OS is freed from any existing masters and
enslaved to the register you've just assigned to. This allows the system
call code to refer to $%OS to try to guess what you want. The system call
will use up to four arguments, provided by registers I<.-$%OS>, I<:-$%OS>,
I<,-$%OS> and I<;-$%OS>, in other words the spot, two spot, tail and hybrid
register with the same number as whatever you last assigned to. This is
called "call by vague resemblance to the last assignment" and, to our
knowledge, no other language has ever used this style of parameter passing.

To use, you do something like "(666) DO .5 <- #1" which would execute
syscall #1, assuming %OS has the value #666. This particular example would
store the version number of CLC-INTERCAL in ,5.

=item %PS

Program symbol (%14)

Determines where the compiler starts when parsing a program. Changing the
value of this register at runtime can be a great obfuscation tool. See also
I<%IS>.

=item %RT

Read Type (%2)

Determines how numeric READ OUT produces its output. Assigning a number to
this register causes the corresponding style to be selected; assigning a
I<MUL> causes a symbolic lookup to determine the style number. See
L<Language::INTERCAL::ReadNumbers>.

=item %SP

SPlat (%19)

This register contains the code of the last splat, just like the '*'
expression. Assigning to it, however, does not cause a splat, but will
trigger any events depending on splats. This register is intended for
internal use by the compiler; programs should use I<SPL> instead.

=item %SS

Space symbol (%13)

The compiler will automatically ignore anything matched by this symbol. If
the Whitespace extension is installed, anything matched by this symbol is
passed to the Whitespace compiler. See the documentation which comes with
the Whitespace extension. Changing the value of this register at runtime can
be a great obfuscation tool.

=item %TM

Trace Mode (%9)

If %TM is zero, the program is not traced. If it is #1 the program will send
bytecode trace information to @9. Assigning any other value to %TM is an
error.

=item %WT

Write Type (%1)

Determines how numeric WRITE IN behaves. The default value of #0 corresponds
to the standard, traditional form; the value #1 enables wimp mode for input.
Any other value is invalid.

=back

=head2 SHARK FIN REGISTER

Shark fin registers hold tail arrays which can be useful for the runtime
and the compilers.

=over 4

=item ^AV

Argument Vector (^1)

The runtime initialises it with any command-line arguments provided to the
program.

=item ^EV

Environment Vector (^2)

The runtime initialises it with environment variables available when the
program starts.

=back

=head2 EXTRA MEANING FOR WHIRLPOOL REGISTERS

Selected whirlpool registers contain some extra data determining a filehandle.
For example, to send some output to the standard splat filehandle and then
restore normal behaviour (output to standard read) one could say:

    DO READ OUT @3 + ,1 + @1

Similarly, if such registers appear in a WRITE IN they change the filehandle
for input operations.

Using of whirlpool registers with other numbers is currently an error: in
future, they will enable virtual I/O using classes.

=over 4

=item @0 (@OR)

Overload Register

This register cannot be used directly. During overload, it is temporarily
enslaved to the register being overloaded, so it is possible to refer to
this register's owner (I<OWN> I<1> I<WHP> I<0>, corresponding to I<$@0>)
within the second argument of I<OVR> or I<OVM> opcodes.

=item @1 (@OWFH)

Object's write filehandle

Used by default for all WRITE IN statements. Defaults to the standard write
filehandle, which your operating system may call "standard input".

=item @2 (@ORFH)

Object's read filehandle

Used by default for all READ OUT statements. Defaults to the standard read
filehandle, which your operating system may call "standard output"

=item @3 (@OSFH)

Object's splat filehandle

Used by default for the text produced by a splat. Defaults to the standard
splat filehandle, which your operating system may call "standard error".

=item @7 (@SNFH)

Null filehandle

If you READ OUT to @7 nothing happens. If you WRITE IN from @7 you get an
empty write (for numeric writes, a value of #0, for non-numeric writes an
empty array).

=item @9 (@TRFH)

Trace filehandle

Trace output, if enabled by I<%TM>, is sent here. Defaults to the standard
splat filehandle, which your operating system may call "standard error"

=back

=head1 SEE ALSO

A qualified psychiatrist

=head1 AUTHOR

Claudio Calvelli - intercal (at) sdf.lonestar.org

