package Language::INTERCAL::Splats;

# Splats and error messages

# This file is part of CLC-INTERCAL

# Copyright (c) 2006 Claudio Calvelli, all rights reserved.

# CLC-INTERCAL is copyrighted software. However, permission to use, modify,
# and distribute it is granted provided that the conditions set out in the
# licence agreement are met. See files README and COPYING in the distribution.


use strict;
use vars qw($PERVERSION);
$PERVERSION = "CLC-INTERCAL INTERCAL/Splats.pm 1.-94.-4";

use Carp;

use Language::INTERCAL::Exporter '1.-94.-4';

use vars qw(@EXPORT @EXPORT_OK %EXPORT_TAGS);

@EXPORT_OK = qw(
    splatnumber splatname splatdescription faint
    SP_ARRAY SP_ASSIGN SP_BASE SP_BCMATCH SP_BUG SP_CHARSET SP_CIRCULAR
    SP_CLASS SP_CLASSWAR SP_COMEFROM SP_COMMENT SP_CONTEXT SP_CONVERT
    SP_CREATION SP_DIGITS SP_DIVIDE SP_EARLY SP_EVENT SP_EVOLUTION
    SP_FALL_OFF SP_FORBIDDEN SP_FREE SP_HIDDEN SP_HOLIDAY SP_ILLEGAL
    SP_INDIGESTION SP_INTERNAL SP_INVALID SP_INVARRAY SP_INVCLASS
    SP_INVLABEL SP_INVSPLAT SP_IOTYPE SP_ISARRAY SP_ISCLASS SP_ISNUMBER
    SP_JUNK SP_LECTURE SP_NEXTING SP_NOARRAY SP_NOASSIGN SP_NOBELONG
    SP_NOCLASS SP_NOCURRICULUM SP_NODIM SP_NONUMBER SP_NOOWNER
    SP_NOREGISTER SP_NORESUME SP_NOSPECIAL SP_NOSTUDENT SP_NOSUCHCHAR
    SP_NOSUCHLABEL SP_NOSYSCALL SP_NOTCLASS SP_NOTHREAD SP_NUMBER SP_OVREG
    SP_OWNER SP_QUANTUM SP_READ SP_REGISTER SP_REGNUM SP_RESUME SP_ROMAN
    SP_SPECIAL SP_SPLAT SP_SPOTS SP_SUBSCRIPT SP_SUBSIZE SP_SWAP SP_SYMBOL
    SP_SYSCALL SP_THREESPOT SP_TODO SP_TOOMANYLABS SP_UBUG
);

%EXPORT_TAGS = (
    SP => [qw(
	faint
	SP_ARRAY SP_ASSIGN SP_BASE SP_BCMATCH SP_BUG SP_CHARSET SP_CIRCULAR
	SP_CLASS SP_CLASSWAR SP_COMEFROM SP_COMMENT SP_CONTEXT SP_CONVERT
	SP_CREATION SP_DIGITS SP_DIVIDE SP_EARLY SP_EVENT SP_EVOLUTION
	SP_FALL_OFF SP_FORBIDDEN SP_FREE SP_HIDDEN SP_HOLIDAY SP_ILLEGAL
	SP_INDIGESTION SP_INTERNAL SP_INVALID SP_INVARRAY SP_INVCLASS
	SP_INVLABEL SP_INVSPLAT SP_IOTYPE SP_ISARRAY SP_ISCLASS SP_ISNUMBER
	SP_JUNK SP_LECTURE SP_NEXTING SP_NOARRAY SP_NOASSIGN SP_NOBELONG
	SP_NOCLASS SP_NOCURRICULUM SP_NODIM SP_NONUMBER SP_NOOWNER
	SP_NOREGISTER SP_NORESUME SP_NOSPECIAL SP_NOSTUDENT SP_NOSUCHCHAR
	SP_NOSUCHLABEL SP_NOSYSCALL SP_NOTCLASS SP_NOTHREAD SP_NUMBER
	SP_OVREG SP_OWNER SP_QUANTUM SP_READ SP_REGISTER SP_REGNUM
	SP_RESUME SP_ROMAN SP_SPECIAL SP_SPLAT SP_SPOTS SP_SUBSCRIPT
	SP_SUBSIZE SP_SWAP SP_SYMBOL SP_SYSCALL SP_THREESPOT SP_TODO
	SP_TOOMANYLABS SP_UBUG
    )],
);

my %splatbyname = (
    ARRAY => 280,
    ASSIGN => 277,
    BASE => 8,
    BCMATCH => 578,
    BUG => 774,
    CHARSET => 10,
    CIRCULAR => 818,
    CLASS => 254,
    CLASSWAR => 603,
    COMEFROM => 555,
    COMMENT => 0,
    CONTEXT => 398,
    CONVERT => 444,
    CREATION => 816,
    DIGITS => 534,
    DIVIDE => 662,
    EARLY => 999,
    EVENT => 751,
    EVOLUTION => 815,
    FALL_OFF => 633,
    FORBIDDEN => 796,
    FREE => 511,
    HIDDEN => 436,
    HOLIDAY => 799,
    ILLEGAL => 997,
    INDIGESTION => 664,
    INTERNAL => 634,
    INVALID => 69,
    INVARRAY => 251,
    INVCLASS => 253,
    INVLABEL => 131,
    INVSPLAT => 457,
    IOTYPE => 9,
    ISARRAY => 243,
    ISCLASS => 244,
    ISNUMBER => 438,
    JUNK => 128,
    LECTURE => 699,
    NEXTING => 123,
    NOARRAY => 245,
    NOASSIGN => 752,
    NOBELONG => 512,
    NOCLASS => 250,
    NOCURRICULUM => 823,
    NODIM => 241,
    NONUMBER => 701,
    NOOWNER => 513,
    NOREGISTER => 437,
    NORESUME => 621,
    NOSPECIAL => 257,
    NOSTUDENT => 822,
    NOSUCHCHAR => 109,
    NOSUCHLABEL => 129,
    NOSYSCALL => 660,
    NOTCLASS => 242,
    NOTHREAD => 369,
    NUMBER => 281,
    OVREG => 252,
    OWNER => 514,
    QUANTUM => 666,
    READ => 432,
    REGISTER => 433,
    REGNUM => 70,
    RESUME => 632,
    ROMAN => 4,
    SPECIAL => 247,
    SPLAT => 456,
    SPOTS => 274,
    SUBSCRIPT => 276,
    SUBSIZE => 279,
    SWAP => 445,
    SYMBOL => 535,
    SYSCALL => 661,
    THREESPOT => 702,
    TODO => 1,
    TOOMANYLABS => 130,
    UBUG => 775,
);

my %splats = (
    0 => ['COMMENT', '%'],
    1 => ['TODO', 'Not implemented: %'],
    4 => ['ROMAN', 'Unknown read type for Roman numerals: %'],
    8 => ['BASE', 'Base must be between 2 and 7 (got %)'],
    9 => ['IOTYPE', 'Invalid I/O Type: %'],
    10 => ['CHARSET', 'Invalid character set: %'],
    69 => ['INVALID', 'Invalid bytecode (%) for %'],
    70 => ['REGNUM', 'Invalid register number "%"'],
    109 => ['NOSUCHCHAR', 'Invalid character (%) for %'],
    123 => ['NEXTING', 'Program attempted more than % levels of NEXTing'],
    128 => ['JUNK', 'Cannot use JUNK in this grammar'],
    129 => ['NOSUCHLABEL', 'Could not find label %'],
    130 => ['TOOMANYLABS', 'Cannot decide between % instances of label %'],
    131 => ['INVLABEL', 'Invalid label %'],
    241 => ['NODIM', 'Array not dimensioned'],
    242 => ['NOTCLASS', 'Non-class value used as class'],
    243 => ['ISARRAY', 'Array register used as value'],
    244 => ['ISCLASS', 'Class register used as value'],
    245 => ['NOARRAY', 'Non-array register used as array'],
    247 => ['SPECIAL', 'Attempt to use special register %'],
    250 => ['NOCLASS', 'Invalid value % assigned to class'],
    251 => ['INVARRAY', 'Invalid value in array element: %'],
    252 => ['OVREG', 'Cannot use overload register %'],
    253 => ['INVCLASS', 'Invalid lecture: %'],
    254 => ['CLASS', 'Invalid subject: %'],
    257 => ['NOSPECIAL', 'Normal register used as special'],
    274 => ['SPOTS', 'Number % too large for %'],
    276 => ['SUBSCRIPT', 'Invalid subscript %: %'],
    277 => ['ASSIGN', 'Impossible assignment (base %): cannot find #X such that #%X is #%'],
    279 => ['SUBSIZE', 'Invalid number of subscripts: % provided, % required'],
    280 => ['ARRAY', 'Invalid array: %'],
    281 => ['NUMBER', 'Invalid number: %'],
    369 => ['NOTHREAD', 'Thread % does not exist'],
    398 => ['CONTEXT', 'Invalid context: %'],
    432 => ['READ', 'Not suitable for %'],
    433 => ['REGISTER', 'Not a valid register: %'],
    436 => ['HIDDEN', 'Register % stashed away too well'],
    437 => ['NOREGISTER', 'Cannot % numbers'],
    438 => ['ISNUMBER', 'Numbers cannot %'],
    444 => ['CONVERT', 'Cannot convert % to %'],
    445 => ['SWAP', 'Cannot swap % and %'],
    456 => ['SPLAT', 'No splat'],
    457 => ['INVSPLAT', 'Invalid splat %'],
    511 => ['FREE', 'Register % is not a slave'],
    512 => ['NOBELONG', 'Register % does not belong to register %'],
    513 => ['NOOWNER', 'Slave % does not have % owners (just %)'],
    514 => ['OWNER', 'Invalid owner number: %'],
    534 => ['DIGITS', 'Wrong number of digits for base %: %'],
    535 => ['SYMBOL', 'Invalid symbol: %'],
    555 => ['COMEFROM', 'Multiple "COME FROM" %'],
    578 => ['BCMATCH', 'Invalid bytecode pattern in %: %'],
    603 => ['CLASSWAR', 'Class war between % and %'],
    621 => ['NORESUME', 'Pointless RESUME'],
    632 => ['RESUME', 'Program terminated via RESUME'],
    633 => ['FALL_OFF', 'Falling off the edge of the program'],
    634 => ['INTERNAL', 'Internal error: %'],
    660 => ['NOSYSCALL', 'Undefined system call %'],
    661 => ['SYSCALL', 'Missing system call number'],
    662 => ['DIVIDE', 'Unary division by zero'],
    664 => ['INDIGESTION', 'Program is too large'],
    666 => ['QUANTUM', '% does not have a quantum version'],
    699 => ['LECTURE', 'Not in a lecture'],
    701 => ['NONUMBER', 'Value written in is not a number: %'],
    702 => ['THREESPOT', 'Value written in is larger than two spots'],
    751 => ['EVENT', 'Invalid event: BODY WHILE CONDITION'],
    752 => ['NOASSIGN', 'Cannot assign %: %'],
    774 => ['BUG', 'Compiler error'],
    775 => ['UBUG', 'Unexplainable compiler error'],
    796 => ['FORBIDDEN', '% is forbidden in INTERCAL-1972'],
    799 => ['HOLIDAY', 'No class teaches subjects %'],
    815 => ['EVOLUTION', 'Creation not allowed: %'],
    816 => ['CREATION', 'CREATE statement misconfiguration: %'],
    818 => ['CIRCULAR', 'Circular reasoning in %'],
    822 => ['NOSTUDENT', 'Register % is not a student'],
    823 => ['NOCURRICULUM', 'Subject % is not in %\'s curriculum'],
    997 => ['ILLEGAL', 'Illegal operator % for base %'],
    999 => ['EARLY', 'Lecture at % is too early'],
);

sub SP_ARRAY () { 280 }
sub SP_ASSIGN () { 277 }
sub SP_BASE () { 8 }
sub SP_BCMATCH () { 578 }
sub SP_BUG () { 774 }
sub SP_CHARSET () { 10 }
sub SP_CIRCULAR () { 818 }
sub SP_CLASS () { 254 }
sub SP_CLASSWAR () { 603 }
sub SP_COMEFROM () { 555 }
sub SP_COMMENT () { 0 }
sub SP_CONTEXT () { 398 }
sub SP_CONVERT () { 444 }
sub SP_CREATION () { 816 }
sub SP_DIGITS () { 534 }
sub SP_DIVIDE () { 662 }
sub SP_EARLY () { 999 }
sub SP_EVENT () { 751 }
sub SP_EVOLUTION () { 815 }
sub SP_FALL_OFF () { 633 }
sub SP_FORBIDDEN () { 796 }
sub SP_FREE () { 511 }
sub SP_HIDDEN () { 436 }
sub SP_HOLIDAY () { 799 }
sub SP_ILLEGAL () { 997 }
sub SP_INDIGESTION () { 664 }
sub SP_INTERNAL () { 634 }
sub SP_INVALID () { 69 }
sub SP_INVARRAY () { 251 }
sub SP_INVCLASS () { 253 }
sub SP_INVLABEL () { 131 }
sub SP_INVSPLAT () { 457 }
sub SP_IOTYPE () { 9 }
sub SP_ISARRAY () { 243 }
sub SP_ISCLASS () { 244 }
sub SP_ISNUMBER () { 438 }
sub SP_JUNK () { 128 }
sub SP_LECTURE () { 699 }
sub SP_NEXTING () { 123 }
sub SP_NOARRAY () { 245 }
sub SP_NOASSIGN () { 752 }
sub SP_NOBELONG () { 512 }
sub SP_NOCLASS () { 250 }
sub SP_NOCURRICULUM () { 823 }
sub SP_NODIM () { 241 }
sub SP_NONUMBER () { 701 }
sub SP_NOOWNER () { 513 }
sub SP_NOREGISTER () { 437 }
sub SP_NORESUME () { 621 }
sub SP_NOSPECIAL () { 257 }
sub SP_NOSTUDENT () { 822 }
sub SP_NOSUCHCHAR () { 109 }
sub SP_NOSUCHLABEL () { 129 }
sub SP_NOSYSCALL () { 660 }
sub SP_NOTCLASS () { 242 }
sub SP_NOTHREAD () { 369 }
sub SP_NUMBER () { 281 }
sub SP_OVREG () { 252 }
sub SP_OWNER () { 514 }
sub SP_QUANTUM () { 666 }
sub SP_READ () { 432 }
sub SP_REGISTER () { 433 }
sub SP_REGNUM () { 70 }
sub SP_RESUME () { 632 }
sub SP_ROMAN () { 4 }
sub SP_SPECIAL () { 247 }
sub SP_SPLAT () { 456 }
sub SP_SPOTS () { 274 }
sub SP_SUBSCRIPT () { 276 }
sub SP_SUBSIZE () { 279 }
sub SP_SWAP () { 445 }
sub SP_SYMBOL () { 535 }
sub SP_SYSCALL () { 661 }
sub SP_THREESPOT () { 702 }
sub SP_TODO () { 1 }
sub SP_TOOMANYLABS () { 130 }
sub SP_UBUG () { 775 }

sub faint {
    @_ >= 1 or croak "Usage: faint(NUM, ARGS)";
    die splatdescription(@_) . "\n";
}

sub splatnumber {
    @_ == 1 or croak "Usage: splatnumber(SPLATNAME)";
    my $s = shift;
    exists $splatbyname{$s} ? $splatbyname{$s} : -1;
}

sub splatname {
    @_ == 1 or croak "Usage: splatname(SPLAT)";
    my $s = shift;
    exists $splats{$s} ? $splats{$s}[0] : undef;
}

sub splatdescription {
    @_ >= 1 or croak "Usage: splatdescription(SPLAT, ARGS)";
    my $s = shift;
    $s %= 1000;
    return 'Unknown splat code' if ! exists $splats{$s};
    my $desc = $splats{$s}[1];
    $desc =~ s/%/shift || '?'/ge;
    $desc .= " (?" . join(' ', @_) . "?)" if @_;
    sprintf("*%03d %s", $s, $desc);
}

1;

__END__

=pod

=head1 TITLE

Language::INTERCAL::Splats - errors

=head1 DESCRIPTION

Execution of I<CLC-INTERCAL> program can produce many errors, one of
the most common is attempting to execute a comment. Errors are
reported using a I<splat>, consisting of an error code and an error
message. The splat code is also available in the special expression
I<*> after the error occurred: this is only useful when the
program is multithreaded (another thread produced the splat) or
within events, as producing a splat is always fatal and causes the
program to terminate.

When printing a splat, the format will always be:

   *nnn message

where nnn is the splat code. However, the list below shows the splat
code as Snnn instead, to prevent the various pod processors from
renumbering the list. This is a bug with the design of pod, on which
we have no control. Even using *nnn causes strange renumberings or
manifest other bugs in the various I<pod2xxx> programs.

=over 5

=item S0

I<invalid statement>

This error code is used by the compiler when it does not have a more
appropriate error code. If you try to execute the statement, the error
message will be the statement itself.

=item S1

Not implemented: I<description>

The requested compiler or runtime feature has not been written.

=item S4

Unknown read type for Roman numerals: I<type>

The compiler attempted to specify an invalid default representation for
Roman numerals. Numeric output cannot work.

=item S8

Base must be between 2 and 7 (got I<base>)

The program assigned an invalid value to %BA

=item S9

Invalid I/O Type: I<type>

The compiler attempted to set the default input/output mechanism (usually
one of CLC or C). Binary and alphanumeric I/O cannot work.

=item S10

Invalid character set: I<name>

The compiler attempted to set an invalid default character set. Try using
one of ASCII, Baudot, EBCDIC or Hollerith.

=item S69

Invalid bytecode (I<code>) for I<item>

An expression or overload contains bytecode designed for use in statements;
or conversely a statement contains an expression bytecode where the
interpreter doesn't expect it.

=item S70

Invalid register number "I<register>"

The program used something like I<.0> or I<.65536>. Note that this may be an
unintentional consequence of assigning to a constant.

=item S109

Invalid character (I<code>) for I<charset>

The conversion between character sets just could not cope with this. This
splat is currently generated when converting to Baudot or Hollerith, when a
character does not exist in that character set; or when converting from
Hollerith when an overpunch just cannot be represented.

=item S123

Program attempted more than I<number> levels of NEXTing

INTERCAL programs aren't supposed to do that.

=item S128

Cannot use JUNK in this grammar

A grammar used the special predefined symbol JUNK but did not provide enough
evidence that it knows what to do with it.

=item S129

Could not find label I<label>

A NEXT or STUDY refers to an undefined label.

=item S130

Cannot decide between I<number> instances of label I<label>

A NEXT or STUDY refers to a label which is multiply defined. This may be an
unintended effect of using computed labels.

=item S131

Invalid label I<label>

A NEXT or STUDY refers to a label which is not valid, for example zero or
too large.

=item S241

Array not dimensioned

Subscripts have been applied to an array register which has not been
dimensioned; or they have been applied to a non-array register.

=item S242

Non-class value used as class

A number or array was used where a class would be better.

=item S243

Array register used as value

An array was used where a number would be more appropriate.

=item S244

Class register used as value

A value was assigned to a class register, or a class register was used for
arithmetic.

=item S245

Non-array register used as array

A non-array register (or a simple value) was used where the system would
like an array.

=item S247

Attempt to use special register I<name>

The given special register does not accept assignments. This may indicate a
bug in CLC-INTERCAL.

=item S250

Invalid value I<value> assigned to class

The program assigned a value to a class register. This is not a good idea.

=item S251

Invalid value in array element: I<reason>

An array is being created, but assignment to one of its elements provided an
invalid value, such as another array.

=item S252

Cannot use overload register I<name>

The overload register can only be used within an overload expression, and
only to find out its owner. Any other use is forbidden.

=item S253

Invalid lecture: I<reason>

A lecture is being defined (e.g. by STUDY) but something went wrong.

=item S254

Invalid subject: I<reason>

Class registers can be used only in special ways. This wasn't one of them.

=item S257

Normal register used as special

Somehow the program managed to use a normal register (spot, two spot, tail,
hybrid, whirlpool) as a special (double-oh-seven, shark fin). May indicate a
compiler bug.

=item S274

Number I<value> too large for I<number of spots>

An arithmetic calculation intended for the given number of spots resulted in
a number too large to fit there.

=item S276

Invalid subscript I<value>: I<reason>

A subscript is zero or greater than the size of the array.

=item S277

Impossible assignment (base I<base>): cannot find #X such that #I<operation>X is #I<result>

Assigning to a unary operator is sometimes impossible.

=item S279

Invalid number of subscripts: I<number> provided, I<number> required

The number of subscripts supplied to an array is wrong.

=item S280

Invalid array: I<reason>

Program attempted to create an array (for example by dimensioning a tail or
hybrid register) but something went wrong.

=item S281

Invalid number: I<reason>

The program attempted to use something weird where a number would have been
a good idea instead.

=item S369

Thread I<thread ID> does not exist

An attempt was made to duplicate or kill a thread which does not exist; it
may have already terminated or it has never been created.

=item S398

Invalid context: I<explanation]>

Something was used in the wrong place.

=item S432

Not suitable for I<operation>

The program attempted to READ OUT or WRITE IN something which is not
suitable for I/O. You can use numbers, arrays or classes but this was none
of the above.

=item S433

Not a valid register: I<register>

The program used something hoping it would work as a register. It didn't.

=item S436

Register I<name> stashed away too well

The program RETRIEVEd more than it SHASHed.

=item S437

Cannot I<operation> numbers

The program attempted to STASH or RETRIEVE a number rather than a register.
This may be an unintended effect of overloading.

=item S438

Numbers cannot I<operation>

The program did something like #1 LEARNS or #2 GRADUATES. In other words, a
number was used where one would like a register. This may be an unintended
effect of overloading.

=item S444

Cannot convert I<opcode> to I<opcode>

The two opcodes are not compatible; it won't work.

=item S445

Cannot swap I<opcode> and I<opcode>

The two opcodes are not compatible; it won't work.

=item S456

No splat

Program used the splat (*) operator to find out the code of the last splat,
but the program hasn't splatted yet.

=item S457

Invalid splat I<number>

Invalid splat. This is likely to happen when you assign to the '*'
expression.

=item S511

Register I<name> is not a slave

The program requested the register's owner but the register is not enslaved
to anything.

=item S512

Register I<slave> does not belong to register I<master>

The program attempted to FREE without ENSLAVing first.

=item S513

Slave I<name> does not have I<number> owners (just I<number>)

The program requested the n-th owner of a register but the register does not
have so many owners.

=item S514

Invalid owner number: I<value>

The program requested an invalid owner number, probably zero.

=item S534

Wrong number of digits for base I<base>: I<number>

An arithmetic calculation in the specified base resulted in a number with
too many digits.

=item S535

Invalid symbol: I<name>

The program used I<name> as a symbol, but this is not OK.

=item S555

Multiple "COME FROM" I<label>

The program has multiple COME FROMs all pointing at the same statement, and
multithreading is disabled.

=item S578

Invalid bytecode pattern in I<name>: I<problem>

A bytecode pattern specified in an optimiser contained a problem.

=item S603

Class war between I<class> and I<class>

Two classes teach the same subject and a register is enrolled in both. Can't
decide which lecture to use.

=item S621

Pointless RESUME

A program attempted to RESUME #0.

=item S632

Program terminated via RESUME

A program RESUMEd more than NEXTed.

=item S633

Falling off the edge of the program

The program tried to execute past its end. Try adding a PLEASE GIVE UP
somewhere.

=item S634

Internal error: I<explanation>

The unthinkable happened and there appears to be a bug in CLC-INTERCAL.
Probably an illusion but do submit a bug report. Include your program (if
not too embarrassed) and the precise text of the error message (it does
contain clues to the developers); also include any other useful information,
including, but not limited to, compiler options used, perversion of
CLC-INTERCAL, any patches and/or modifications you have applied to the
compiler, and what colour was selected by the system to display the error
message.

=item S660

Undefined system call I<number>

Program made a system call but the system did not know what to do about it.

=item S661

Missing system call number

Program attempted to make a system call but did not provide a system call
number.

=item S662

Unary division by zero

Program attempted unary division by zero.

=item S664

Program is too large

Either the source or the comnpiled code exceeded some built-in arbitrary
limits.

=item S666

I<statement> does not have a quantum version

Program attempted to run a statement in quantum mode, but the statement
cannot do that.

=item S699

Not in a lecture

Cannot FINISH LECTURE unless there is one.

=item S701

Value written in is not a number: I<value>

You typed something like ZERRO instead of ZERO. Or perhaps you used a
language which is not understood by CLC-INTERCAL.

=item S702

Value written in is larger than two spots

You typed too many digits.

=item S751

Invalid event: BODY WHILE CONDITION

Before you can use it you must CONVERT it to a CONDITION WHILE BODY (or you
must SWAP them).

=item S752

Cannot assign I<value>: I<reason>

The program attempted to assign an invalid value to a register.

=item S774

Compiler error

Program encountered a compiler error.

=item S775

Unexplainable compiler error

Program encountered a compiler error, even though the "compiler error" has
been disabled by a compiler option.

=item S796

I<operation> is forbidden in INTERCAL-1972

You asked for a compiler compatible with INTERCAL-1972 but used some
extensions, such as binary I/O. Select CLC-INTERCAL native mode or
compatibility with C-INTERCAL to use these things.

=item S799

No class teaches subjects I<number>

Maybe it's some kind of holiday?

=item S815

Creation not allowed: I<reason>

A CREATE (or DESTROY) statement attempted something illegal.

=item S816

CREATE statement misconfiguration: I<reason>

The production contained some invalid code.

=item S818

Circular reasoning in I<symbol>

Recursion is acceptable, but here the problem is that the symbol is defined
as being identical to itself, which is true but not useful.

=item S822

Register I<name> is not a student

The program attempted to LEARN something, but the register has not enrolled
in a class.

=item S823

Subject I<number> is not in I<register>'s curriculum

Program attempted to LEARN something, but the register is not in the right
class.

=item S997

Illegal operator I<name> for base I<base>

Maybe you forgot to include the appropriate compiler option for the correct
base? Or used the wrong suffix? Or you overloaded the base and so can't tell
what base you are using for calculations?

=item S999

Lecture at I<label> is too early

Program attempted to define a lecture before 1000. That's way too early.

=back

=head1 SEE ALSO

A qualified psychiatrist

=head1 AUTHOR

Claudio Calvelli - intercal (at) sdf.lonestar.org

